@@ -48,9 +48,12 @@
 #'   start = mean_dp)
 #' summary(m_nlme)
 #' plot(augPred(m_nlme, level = 0:1), layout = c(3, 1))
-#' # augPred does not seem to work on fits with more than one state
+#' # augPred does not work on fits with more than one state
 #' # variable
-#'
+#' #
+#' # The procedure is greatly simplified by the nlme.mmkin function
+#' f_nlme <- nlme(f)
+#' plot(f_nlme)
 #' @return A function that can be used with nlme
 #' @export
 nlme_function <- function(object) {
@@ -130,12 +133,14 @@ nlme_function <- function(object) {
 #' @export
 mean_degparms <- function(object, random = FALSE) {
   if (nrow(object) > 1) stop("Only row objects allowed")
-  degparm_mat_trans <- sapply(object, parms, transformed = TRUE)
-  mean_degparm_names <- setdiff(rownames(degparm_mat_trans), names(object[[1]]$errparms))
-  fixed <- apply(degparm_mat_trans[mean_degparm_names, ], 1, mean)
+  parm_mat_trans <- sapply(object, parms, transformed = TRUE)
+  mean_degparm_names <- setdiff(rownames(parm_mat_trans), names(object[[1]]$errparms))
+  degparm_mat_trans <- parm_mat_trans[mean_degparm_names, , drop = FALSE]
+  fixed <- apply(degparm_mat_trans, 1, mean)
   if (random) {
-    degparm_mat_trans[mean_degparm_names, ]
-    random <- t(apply(degparm_mat_trans[mean_degparm_names, ], 2, function(column) column - fixed))
+    random <- t(apply(degparm_mat_trans[mean_degparm_names, , drop = FALSE], 2, function(column) column - fixed))
+    # If we only have one parameter, apply returns a vector so we get a single row
+    if (nrow(degparm_mat_trans) == 1) random <- t(random)
     rownames(random) <- levels(nlme_data(object)$ds)
     return(list(fixed = fixed, random = list(ds = random)))
   } else {
