@@ -1,4 +1,4 @@
-if(getRversion() >= '2.15.1') utils::globalVariables(c("type", "variable", "observed"))
+utils::globalVariables(c("type", "variable", "observed"))
 
 #' Plot the observed data and the fitted model of an mkinfit object
 #'
@@ -91,7 +91,7 @@ if(getRversion() >= '2.15.1') utils::globalVariables(c("type", "variable", "obse
 #' @export
 plot.mkinfit <- function(x, fit = x,
   obs_vars = names(fit$mkinmod$map),
-  xlab = "Time", ylab = "Observed",
+  xlab = "Time", ylab = "Residue",
   xlim = range(fit$data$time),
   ylim = "default",
   col_obs = 1:length(obs_vars),
@@ -277,7 +277,7 @@ plot.mkinfit <- function(x, fit = x,
     if (show_residuals) {
       mkinresplot(fit, obs_vars = row_obs_vars, standardized = standardized,
         pch_obs = pch_obs[row_obs_vars], col_obs = col_obs[row_obs_vars],
-        legend = FALSE, frame = frame)
+        legend = FALSE, frame = frame, xlab = xlab)
     }
 
     # Show error model plot if requested
